#include <Evas.h>
#include <Ecore.h>
#include <Ecore_Evas.h>
#include <math.h>

Evas        *evas = NULL;
Ecore_Evas  *ecore_evas = NULL;

Evas_Object *background = NULL;
Evas_Object *clip = NULL;
double anim_start = 0.0;

static void
callback_delete_request(Ecore_Evas *ee)
{
  ecore_main_loop_quit();
}

static void
callback_resize(Ecore_Evas *ee)
{
  Evas_Coord w, h;
  
  evas_output_viewport_get(evas, NULL, NULL, &w, &h);
  evas_object_resize(background, w, h);
}

static int
callback_animate(void *data)
{
  Evas_Object *o = data;
  Evas_Coord w, h;
  double t = ecore_loop_time_get() - anim_start;
  double x, y;
  int delta = (int)data & 0xff;
  
  evas_output_viewport_get(evas, NULL, NULL, &w, &h);
  x = cos((t + (delta / 100.0)) * 2) * 100;
  y = sin((t + (delta / 100.0)) * 2) * 100;
  evas_object_move(o, (w / 2) + x - 50, (h / 2) + y - 50);
  return 1;
}

static int
callback_animate_clip(void *data)
{
  Evas_Object *o = data;
  double t = ecore_loop_time_get() - anim_start;
  int a;
  
  a = ((int)(t * 255.0)) & 0xff;
  evas_object_color_set(o, a, a, a, a);
  return 1;
}

int
main(int argc, char **argv)
{
  Evas_Object *o;
  int i;
  
  ecore_evas_init();
  ecore_evas = ecore_evas_software_x11_new(NULL, 0, 0, 0, 240, 240);
  ecore_evas_callback_delete_request_set(ecore_evas, callback_delete_request);
  ecore_evas_callback_resize_set(ecore_evas, callback_resize);
  ecore_evas_title_set(ecore_evas, "Tut 02");
  ecore_evas_name_class_set(ecore_evas, "main", "tut-02");
  evas = ecore_evas_get(ecore_evas);

  o = evas_object_rectangle_add(evas);
  evas_object_color_set(o, 255, 255, 255, 255);
  evas_object_move(o, 0, 0);
  evas_object_resize(o, 240, 240);
  evas_object_layer_set(o, -1000);
  evas_object_show(o);
  background = o;
  
  o = evas_object_rectangle_add(evas);
  evas_object_color_set(o, 0, 0, 0, 0);
  evas_object_move(o, 20, 20);
  evas_object_resize(o, 200, 200);
  evas_object_show(o);
  ecore_animator_add(callback_animate_clip, o);
  clip = o;

  for (i = 0; i < 100; i++)
    {
      o = evas_object_rectangle_add(evas);
      evas_object_color_set(o, (int)o & 0xff, ((int)o >> 8) & 0xff, ((int)o >> 16) & 0xff, 255);
      evas_object_resize(o, 100, 100);
      evas_object_show(o);
      evas_object_clip_set(o, clip);
      ecore_animator_add(callback_animate, o);
    }
  
  ecore_animator_frametime_set(1.0 / 50.0);
  anim_start = ecore_loop_time_get();
  ecore_evas_show(ecore_evas);

  ecore_main_loop_begin();

  ecore_evas_shutdown();
  return 0;
}
